import os
import sys
import argparse

from .log import logger
from .settings import args


def _dir_maker(dir_):
    if not os.path.exists(dir_):
        logger.info("%s not found, creating local folder" % dir_)
        os.makedirs(dir_)

if args.log_to_file:
    _dir_maker(settings.args.logs_dir)

_dir_maker(settings.args.database_migration_dir)




from tabulate import tabulate

from .helpers import Utils
from .log import logger, bcolors
from .migration import Migration
from .dbms.database import factory
from .errors import MIGRATIONS_NOT_FOUND


class Info():

    def __init__(self, conf):
        self._database_connection = "%s:%s/%s" % (conf.args.database_host, conf.args.database_port, conf.args.database_name)
        self._migration_dir = conf.args.database_migration_dir
        self._db = factory(conf.args.database_type)(conf)
        self.headers = ["version", "extension", "name", "checksum", "apply_timestamp"]
        self.tablefmt = "psql"

    def run(self):
        logger.info(tabulate(Utils.flatten_migrations(self.get_table_info()), headers="keys", tablefmt=self.tablefmt))

    def get_table_info(self):
        db_migrations = self._db.get_all_schema_migrations()
        local_migrations = self.get_new_local_migrations(db_migrations)
        return db_migrations + local_migrations

    def get_new_local_migrations(self, db_migrations):
        local_files = Utils.get_local_files()
        if not local_files:
            return []

        new_local_migrations = [self.structure_migration(local_file) for local_file in local_files
                                if local_file not in [db_migration.name for db_migration in db_migrations]]
        return Utils.sort_migrations_list(new_local_migrations)

    def structure_migration(self, name):
        checksum = "%snew%s" % (bcolors.OKGREEN, bcolors.OKBLUE)
        apply_timestamp = "%snew%s" % (bcolors.OKGREEN, bcolors.OKBLUE)
        return Migration.from_name(name, checksum=checksum, apply_timestamp=apply_timestamp)



import os
import argparse
import yaml
import sys

from pyway.version import __version__

# Initialization
LOGO = f"PyWay {__version__}"

parser = argparse.ArgumentParser()
parser.add_argument("--database-migration-dir", help="Database migration directory",
        default=os.environ.get('PYWAY_DATABASE_MIGRATION_DIR', 'resources'))
parser.add_argument("--database-table", help="Database table that stores pyway metadata",
        default=os.environ.get('PYWAY_TABLE', 'public.pyway'))
parser.add_argument("--database-type", help="Database type [postgres|mysql]", default=os.environ.get('PYWAY_TYPE', 'postgres'))

parser.add_argument("--database-host", help="Database host", default=os.environ.get('PYWAY_DATABASE_HOST', 'localhost'))
parser.add_argument("--database-port", help="Database port", default=os.environ.get('PYWAY_DATABASE_PORT', '5432'))
parser.add_argument("--database-name", help="Database name", default=os.environ.get('PYWAY_DATABASE_NAME', 'postgres'))
parser.add_argument("--database-username", help="Database username", default=os.environ.get('PYWAY_DATABASE_USERNAME', 'postgres'))
parser.add_argument("--database-password", help="Database password", default=os.environ.get('PYWAY_DATABASE_PASSWORD', 'password'))

parser.add_argument("-c", "--config", help="Config file", default=os.environ.get('PYWAY_CONFIG_FILE', '.pyway.conf'))
parser.add_argument("-v", "--version", help="Version", action='store_true')
parser.add_argument("--logs-dir", help="Logs directory", default=os.environ.get('PYWAY_LOGS_DIR', 'logs'))
parser.add_argument("--log-to-file", help="Log to file", action='store_true')
parser.add_argument("cmd", nargs="?", help="info|validate|migrate")

args = parser.parse_args()

# Pyway vars that shouldn't change
SQL_MIGRATION_PREFIX = os.environ.get('PYWAY_SQL_MIGRATION_PREFIX', 'V')
SQL_MIGRATION_SEPARATOR = os.environ.get('PYWAY_SQL_MIGRATION_SEPARATOR', '__')
SQL_MIGRATION_SUFFIXES = os.environ.get('PYWAY_SQL_MIGRATION_SUFFIXES', '.sql')

# Display version if it exists
if args.version:
    print(f"Version: {__version__}")
    sys.exit(1)

# If no arg is specified, show help
if not args.cmd:
    parser.print_help()
    sys.exit(1)

# See if there is a config file
if os.path.exists(args.config):
    with open(args.config, "r", encoding='utf-8') as ymlfile:
        cfg = yaml.load(ymlfile, Loader=yaml.FullLoader)

    # Merge config together with args
    if 'database' in cfg:
        if 'type' in cfg['database']:
            args.database_type = cfg['database']['type']
        if 'username' in cfg['database']:
            args.database_username = cfg['database']['username']
        if 'password' in cfg['database']:
            args.database_password = cfg['database']['password']
        if 'database' in cfg['database']:
            args.database_name = cfg['database']['database']
        if 'host' in cfg['database']:
            args.database_host = cfg['database']['host']
        if 'port' in cfg['database']:
            args.database_port = cfg['database']['port']

    if 'logging' in cfg:
        if 'logsdir' in cfg['logging']:
            args.logs_dir = cfg['logging']['logsdir']
        if 'logtofile' in cfg['logging']:
            args.log_to_file = cfg['logging']['logtofile']

    if 'general' in cfg:
        if 'migrationsdir' in cfg['general']:
            args.database_migration_dir = cfg['general']['migrationsdir']
        if 'pywaytable' in cfg['general']:
            args.database_table = cfg['general']['pywaytable']



import sys

from pyway import settings
from pyway.info import Info
from pyway.log import logger
from pyway.migrate import Migrate
from pyway.validate import Validate


def migrate():
    logger.info('Starting migration process...')
    Migrate(settings).run()
    logger.info('Migration completed.')


def validate():
    logger.info('Starting validation process')
    Validate(settings).run()
    logger.info('Validation completed.')


def info():
    logger.info('Gathering info...')
    Info(settings).run()
    print()

def cli():
    logger.info(settings.LOGO)

    if settings.args.cmd == "info":
        info()
    elif settings.args.cmd == "validate":
        validate()
    elif settings.args.cmd == "migrate":
        migrate()
    else:
        logger.error(f"Command '{settings.args.cmd}' not recognized, exiting!")
        sys.exit(1)

if __name__ == '__main__':
    cli()



from distutils.core import setup
from setuptools import find_packages
from codecs import open
from os import path

from pyway.version import __version__


here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# get the dependencies and installs
with open(path.join(here, 'requirements.txt'), encoding='utf-8') as f:
    all_reqs = f.read().split('\n')

install_requires = [x.strip() for x in all_reqs if 'git+' not in x]

setup(
    name='pyway',
    version=__version__,
    description='Pyway is a database versioning and migration tool inspired by Flyway',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/jasondcamp/pyway',
    download_url='https://github.com/jasondcamp/pyway/tarball/' + __version__,
    license='GPL',
    classifiers=[
      'Development Status :: 4 - Beta',
      'Programming Language :: Python :: 3.7',
      'Programming Language :: Python :: 3.8',
      'Programming Language :: Python :: 3.9',
      'Programming Language :: Python :: 3.10',
      'Programming Language :: Python :: 3.11',
    ],
    keywords='',
    packages=find_packages(exclude=['docs', 'tests*']),
    include_package_data=True,
    author='Jason Camp',
    install_requires=install_requires,
    author_email='me@jasoncamp.com'
)
